#! usr/bin/env python

# Hidas' Pyweek 13 entry

import pygame
import sys
import random
import math
import os
import numpy
import time
pygame.init()
size = width, height = 640, 640
screen = pygame.display.set_mode(size)
clock = pygame.time.Clock()
SCROLLOFFSET = 220
CAMERASPEED = 15
def imgload(imagefile, colorkey=[255, 255, 255]):
	try:
		image = pygame.image.load(os.getcwd()+os.sep+'images'+os.sep+imagefile)
		image.set_colorkey(colorkey)
		image.convert()
	except:
		print 'Cannot load image: '+imagefile
	return image
class Cave():
	# A procedurally generated cave. Stores its width and height in tiles, and references the tiles that make it up
	def __init__(self, width, height):
		self.width = width
		self.height = height
		self.tiles = pygame.sprite.Group()
		self.playerpos = [0, 0]
		self.deadlies = pygame.sprite.Group()
	def update(self, worldmovement):
		global screen
		self.deadlies.update(worldmovement)
		self.tiles.update(worldmovement)
		screenrect = screen.get_rect()
		for tile in self.tiles:
			if tile.rect.colliderect(screenrect):
				screen.blit(tile.image, tile.rect)
		for deadly in self.deadlies:
			if deadly.rect.colliderect(screenrect):
				screen.blit(deadly.image, deadly.rect)
		return
class Tile(pygame.sprite.Sprite):
	# Just your average tile. Stores its rect, image, and whether is has been given life.
	def __init__(self, position):
		pygame.sprite.Sprite.__init__(self)
		self.rect = pygame.Rect(position, (32, 32))
		self.haslife = False
		self.image = imgload('tile.png')
	def update(self, worldmovement):
		# Update movement and return. Drawing is handled by the Cave class.
		self.rect = self.rect.move(worldmovement)
		#pygame.draw.rect(screen, [255, 255, 255], self.rect, 2)
		return
class Deadly(Tile):
	def __init__(self, position):
		pygame.sprite.Sprite.__init__(self)
		self.rect = pygame.Rect(position, (32, 32))
		self.image = pygame.Surface((32, 32))
		self.image.fill((255, 10, 0))
		
class Player(pygame.sprite.Sprite):
	# Ho-hum, your average platforming guy.
	def __init__(self, position):
		pygame.sprite.Sprite.__init__(self)
		self.rect = pygame.Rect(position, (17, 17))
		self.rect.topleft = position
		self.imagesright = [imgload('human1.png'), imgload('human2.png')]
		self.imagesleft = [imgload('humanleft1.png'), imgload('humanleft2.png')]
		self.xaccel = 1.5
		self.jumping = False
		self.onGround = False
		self.yaccel = 2.4
		self.jump_timer = 0
		self.velocity = [0, 0]
	def update(self, cave):
		# Update movement, tile collision, and drawing.
		collide = pygame.sprite.spritecollideany(self, cave.deadlies)
		print collide
		if collide != -1:
			print 'ouch'
		self.updateInput()
		self.updateCollision(cave.tiles)
		worldmovement = self.updateMovement()
		self.draw()
		if self.jump_timer > 0:
			self.jump_timer -= 1
			if pygame.key.get_pressed()[pygame.K_UP]:
				self.velocity[1] -= self.yaccel/2
		if self.velocity[1] > 9:
			self.velocity[1] = 9
		return worldmovement
	def updateInput(self):
		keys = pygame.key.get_pressed()
		if keys[pygame.K_RIGHT]:
			self.velocity[0] += self.xaccel
		elif keys[pygame.K_LEFT]:
			self.velocity[0] += -self.xaccel
		else:
			self.velocity[0] /= 2.3
		if self.velocity[0] > 9:
			self.velocity[0] = 9
		elif self.velocity[0] < -9:
			self.velocity[0] = -9
		if keys[pygame.K_UP] and self.onGround == True:
			self.jumping = True
			self.velocity[1] = -self.yaccel*2
			#self.rect.bottom -= 1
			self.jump_timer = 9
			self.onGround = False
		return
	def updateCollision(self, tiles):
		#if self.velocity[0] != 0:
		keys = pygame.key.get_pressed()
		# Moving right collision test
		for tile in tiles:
			if tile.rect.colliderect(self.rect.move([self.velocity[0], 0])):
				if self.rect.centery >= tile.rect.top and self.rect.centery <= tile.rect.bottom:
					if self.rect.centerx <= tile.rect.centerx:
						self.velocity[0] = 0
						self.velocity[1] = 0
						self.onGround = True
						self.rect.right = tile.rect.left
						if keys[pygame.K_UP]:
							self.jumping = True
							self.velocity[1] = -self.yaccel
							self.jump_timer = 4
							self.onGround = False
							self.velocity[0] = -5
					elif self.rect.centerx >= tile.rect.centerx:
						self.velocity[0] = 0
						self.velocity[1] = 0
						self.onGround = True	
						self.rect.left = tile.rect.right
						if keys[pygame.K_UP]:
							self.jumping = True
							self.velocity[1] = -self.yaccel
							self.jump_timer = 4
							self.onGround = False
							self.velocity[0] = 5
					break
		# Vertical collision tests
		#if self.jumping == True:
		if self.velocity[1] < 0:
			for tile in tiles:
				if tile.rect.colliderect(self.rect.move([0, self.velocity[1]])):
					if self.rect.centerx >= tile.rect.left and self.rect.centerx <= tile.rect.right:
						self.rect.top = tile.rect.bottom+1
						self.jumping = False
						self.velocity[1] = 1
						self.jump_timer = 0
						self.onGround = False
						break
		if self.jump_timer == 0:
			self.jumping = False
		if self.velocity[1] >= 0:
			if self.velocity[1] == 0:
				self.velocity[1] = 1
			for tile in tiles:
				if not tile.rect.colliderect(self.rect.move([0, self.velocity[1]])):
					self.onGround = False
				else:
					if self.rect.centerx >= tile.rect.left and self.rect.centerx <= tile.rect.right:
						self.onGround = True
						self.velocity[1] = 0
						self.rect.bottom = tile.rect.top
						break
		else:
			self.onGroudn = False
		if self.onGround == False:
			if self.jumping == False:
				self.onGround = False
				self.velocity[1] += self.yaccel
		return
	def updateMovement(self):
		global screen, SCROLLOFFSET
		worldmovement = [0, 0]
		playermovement = self.velocity[:]
		if self.rect.right > screen.get_width()-SCROLLOFFSET and self.velocity[0] > 0:
			worldmovement[0] = -self.velocity[0]
			playermovement[0] = 0
		if self.rect.left < SCROLLOFFSET and self.velocity[0] < 0:
			worldmovement[0] = -self.velocity[0]
			playermovement[0] =0
		if self.rect.top < SCROLLOFFSET and self.velocity[1] < 0:
			worldmovement[1] = -self.velocity[1]
			playermovement[1] = 0
		if self.rect.bottom > screen.get_height()-SCROLLOFFSET and self.velocity[1] > 0:
			worldmovement[1] = -self.velocity[1]
			playermovement[1] = 0

		self.rect = self.rect.move(playermovement)
		return worldmovement
	def draw(self):
		global screen
		screen.blit(self.imagesright[0], self.rect)
		pygame.draw.rect(screen, [255, 255, 255], self.rect)
def updateGame(player, cave):
	worldmovement = player.update(cave.tiles)
	cave.update(worldmovement)
	return
'''
def createCave(width, height):
	cave = pygame.Surface((width, height))
	cave.fill((0, 0, 0))
	pxarray = pygame.PixelArray(cave)
	running = False
	startpos = [random.randint(0, width), random.randint(0, height)]
	playerpos = startpos[:]
	green = (0, 255, 0)
	times = 0
	cutoff = 2000
	pxarray[startpos] = green
	while times < cutoff:
		direction = random.randint(0, 4)
		if direction == 0:
			try:
				newpos = [startpos[0], startpos[1]-1]
				if pxarray[newpos] == green:
					continue
				pxarray[newpos]=green
				startpos = newpos[:]
			except:
				continue
		if direction == 1:
			try:
				newpos = [startpos[0]+1, startpos[1]]
				if pxarray[newpos] == green:
					continue
				pxarray[newpos]=green
				startpos = newpos[:]
			except:
				continue
		if direction == 2:
			try:
				newpos = [startpos[0], startpos[1]+1]
				if pxarray[newpos] == green:
					continue
				pxarray[newpos] = green
				startpos = newpos[:]
			except:
				continue
		if direction == 3:
			try:
				newpos = [startpos[0]-1, startpos[1]]
				if pxarray[newpos] == green:
					continue
				pxarray[newpos]=green
				startpos = newpos[:]
			except:
				continue
		times += 1
	del pxarray
	screen.blit(cave, [0, 0])
	pygame.display.flip()
	time.sleep(2)
	cave.unlock()
	info = pygame.surfarray.array3d(cave)
	print info
	startpos = [0, 0]
	gamecave = Cave(width, height)
	gamecave.playerpos = playerpos[:]
	for i in info:
		if i == (0, 0, 0):
			gamecave.tiles.add(Tile(startpos[:]))
			startpos[0] += 32
			if startpos[0] >= width*32:
				startpos[0] = 0
				startpos[1] += 32
	print gamecave.tiles
	return gamecave'''
class Grid():
	def __init__(self, width, height):
		self.data = []
		for i in range(0, height):
			self.data.append([0]*width)
	def index(self, x, y):
		if y >= len(self.data):
			return
		return self.data[y][x]
	def replace(self, pos, newitem):
		self.data[pos[0]][pos[1]] = newitem
		return
def createCave(width, height, size):
	cave = Grid(width, height)
	startpos = [0, 0]
	cave.replace(startpos, 2)
	times = 0
	for i in range(0, size):
		direction = random.randint(0, 4)
		if direction == 0:
			newpos = [startpos[0], startpos[1]-1]
			try:
				cave.replace(newpos, 1)
				startpos = newpos[:]
			except:
				continue
		if direction == 1:
			newpos = [startpos[0]+1, startpos[1]]
			try:
				cave.replace(newpos, 1)
				startpos = newpos[:]
			except:
				continue
		if direction == 2:
			newpos = [startpos[0], startpos[1]+1]
			try:
				cave.replace(newpos, 1)
				startpos = newpos[:]
			except:
				continue
		if direction == 3:
			newpos = [startpos[0]-1, startpos[1]]
			try:
				cave.replace(newpos, 1)
				startpos = newpos[:]
			except:
				continue
	startpos = [0, 0]
	gamecave = Cave(width, height)
	for row in cave.data:
		print row
		for tile in row:
			if tile == 0:
				deadly = random.choice([False, False, False, False, False, False, False, False, False, False, True])
				if deadly == False:
					gamecave.tiles.add(Tile(startpos[:]))
				else:
					gamecave.deadlies.add(Deadly(startpos[:]))
			elif tile == 2:
				gamecave.playerpos = startpos[:]
			startpos[0] += 32
		startpos[0] = 0
		startpos[1] += 32
	
	return gamecave 
def moveCamera():
	global CAMERASPEED
	keys = pygame.key.get_pressed()
	movement = [0, 0]
	if keys[pygame.K_d]:
		movement[0] = -CAMERASPEED
	elif keys[pygame.K_a]:
		movement[0] = CAMERASPEED
	if keys[pygame.K_w]:
		movement[1] = CAMERASPEED
	elif keys[pygame.K_s]:
		movement[1] = -CAMERASPEED
	return movement
def main(cave):
	global screen, clock
	player = Player(cave.playerpos)
	while True:
		for e in pygame.event.get():
			if e.type == pygame.QUIT:
				pygame.quit()
				sys.exit()
			elif e.type == pygame.KEYDOWN:
				if e.key == pygame.K_ESCAPE:
					pygame.quit()
					sys.exit()
		screen.fill((3, 3, 3))
		worldmovement= player.update(cave)
		cave.update(worldmovement)
		clock.tick(30)
		pygame.display.flip()

if __name__ == '__main__':
	cave = createCave(100, 100, 10000)
	main(cave)
